#include "camera.h"

void
camera_orthographic_init(Camera* self, vec2 size)
{
	memset(self, '\0', sizeof(Camera));

	self->type = CAMERA_ORTHOGRAPHIC;

	glm_vec2_copy(size, self->orthographic.size);

	self->zoom = 1.0f;
}

void
camera_view_get(Camera* self, mat4 view)
{
	vec3 glPosition;
	vec3 zoom;
	vec3 center;

	glm_mat4_identity(view);

	glPosition[0] = -self->position[0] / (f32)self->orthographic.size[0];
	glPosition[1] = self->position[1] / (f32)self->orthographic.size[1];
	glPosition[2] = 0.0f;

	zoom[0] = self->zoom;
	zoom[1] = self->zoom;
	zoom[2] = 0.0f;

	switch (self->type)
	{
		case CAMERA_ORTHOGRAPHIC:
			glm_mat4_identity(view);
			glm_translate(view, glPosition);
			glm_scale(view, zoom);
			break;
		default:
			break;
	}
}

void
camera_projection_get(Camera* self, mat4 projection)
{
	glm_mat4_identity(projection);

	switch (self->type)
	{
		case CAMERA_ORTHOGRAPHIC:
			glm_ortho
			(
				0.0f,
				self->orthographic.size[0], 
				self->orthographic.size[1], 
				0.0f, 
				CAMERA_ORTHOGRAPHIC_NEAR, 
				CAMERA_ORTHOGRAPHIC_FAR,
				projection
			);
			break;
		default:
			break;
	}
}

void
camera_position_set(Camera* self, vec3 position)
{
	glm_vec3_copy(position, self->position);
}

void
camera_orthographic_window_set(Camera* self, Window* window)
{
	ivec2 size;

	window_size_get(window, size);

	self->orthographic.size[0] = (f32)size[0];
	self->orthographic.size[1] = (f32)size[1];
}
