#include "texture.h"

void
texture_surface_init(Texture* self, SDL_Surface* surface)
{
	ivec2 size;

	size[0] = surface->w;
	size[1] = surface->h;

	texture_init(self, size, surface->pixels);
}

bool
texture_from_path_init(Texture* self, const char* path)
{
	SDL_Surface* surface;

	memset(self, '\0', sizeof(Texture));

	surface = IMG_Load(path);

	if (!surface)
	{
		printf(STRING_SDL_IMAGE_LOAD_ERROR, path, IMG_GetError());
		return false;
	}
		
	printf(STRING_SDL_IMAGE_LOAD_SUCCESS, path);

	texture_surface_init(self, surface);

	SDL_FreeSurface(surface);

	return true;
}

void
texture_init(Texture* self, ivec2 size, void* data)
{
	glGenTextures(1, &self->handle);

	texture_bind(self);

	glm_ivec2_copy(size, self->size);
	
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, self->size[0], self->size[1], 0, GL_RGBA, GL_UNSIGNED_BYTE, data);

	texture_parameter_set(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	texture_parameter_set(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	texture_parameter_set(GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	texture_parameter_set(GL_TEXTURE_MAG_FILTER, GL_NEAREST);

	glGenerateMipmap(GL_TEXTURE_2D);

	self->isInit = true;

	texture_unbind();
}

void
texture_bind(Texture* self)
{
	glBindTexture(GL_TEXTURE_2D, self->handle);
}

void
texture_parameter_set(GLenum pname, GLint parameter)
{
	glTexParameteri(GL_TEXTURE_2D, pname, parameter);
}

void
texture_unbind(void)
{
	glBindTexture(GL_TEXTURE_2D, 0);
}

void
texture_free(Texture* self)
{
	glDeleteTextures(1, &self->handle);
	memset(self, '\0', sizeof(Texture));
}

void
texture_clear(Texture* self)
{
	vec4 color = {1.0f, 0.0f, 0.0f, 1.0f};

	texture_bind(self);

	glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, self->size[0], self->size[1], GL_RGBA, GL_FLOAT, color);
	
	texture_unbind();
}
