#include "vector.h"

static void _vector_realloc(Vector* self);
static void _vector_forward(Vector* self, s32 index);
static void _vector_backward(Vector* self, s32 index);

static void
_vector_realloc(Vector* self)
{
	if (self->count == 0)
	{
		self->data = NULL;
		return;
	}

	self->data = realloc(self->data, vector_size_get(self));
}

static void
_vector_forward(Vector* self, s32 index)
{
	s32 length;

	length = ((self->count - 1) - index);

	for (s32 i = 0; i < length; i++)
	{
		u8* data;
		u8* nextData;
		s32 currentIndex;

		currentIndex = index + i;

		data = (u8*)self->data + (currentIndex * self->itemSize);
		nextData = data + self->itemSize;

		memcpy((void*)data, (void*)nextData, self->itemSize);
	}

}

static void
_vector_backward(Vector* self, s32 index)
{
	s32 length;

	if (self->count == 0)
		return;

	length = ((self->count - 1) - index);

	for (s32 i = 0; i < length; i++)
	{
		u8* data;
		u8* prevData;
		s32 index;

		index = (self->count - 1) - i;

		data = (u8*)self->data + (index * self->itemSize);
		prevData = (u8*)data - self->itemSize;

		memcpy((void*)data, (void*)prevData, self->itemSize);
	}
}

size_t
vector_size_get(Vector* self)
{
	return self->count * self->itemSize;
}

void
vector_init(Vector* self, size_t size)
{
	memset(self, '\0', sizeof(Vector));

	self->itemSize = size;

	_vector_realloc(self);

	memset(self->data, '\0', vector_size_get(self));
}

void
vector_free(Vector* self)
{
	if (self->data)
		free(self->data);

	self->count = 0;
}

void* 
vector_push(Vector* self, void* data)
{
	u8* pointer;

	self->count++;

	_vector_realloc(self);

	pointer = (u8*)self->data + (vector_size_get(self) - self->itemSize);

	if (data)
		memcpy((void*)pointer, data, self->itemSize);
	else
		memset((void*)pointer, '\0', self->itemSize);

	return pointer;
}

void
vector_pop(Vector* self)
{
	self->count--;

	_vector_realloc(self);
}

void
vector_clear(Vector* self)
{
	self->count = 0;

	_vector_realloc(self);
}

void*
vector_insert(Vector* self, void* data, s32 index)
{
	u8* pointer;

	_vector_backward(self, index);

	self->count++;

	_vector_realloc(self);

	pointer = (u8*)self->data + (index * self->itemSize);

	memcpy((void*)pointer, data, self->itemSize);

	return pointer;
}

void 
vector_remove(Vector* self, s32 index)
{
	_vector_forward(self, index);

	vector_pop(self);
}

void*
vector_get(Vector* self, s32 index)
{
	return (void*)((u8*)self->data + (index * self->itemSize));
}

s32
vector_index_get(Vector* self, void* data)
{
	s32 index;

	index = ((u8*)data - (u8*)self->data) / self->itemSize;

	return index;
}

void
vector_remove_from_data(Vector* self, void* data)
{
	for (s32 i = 0; i < (s32)self->count; i++)
	{
		void* pointer;
		s32 result;

		pointer = vector_get(self, i);

		result = memcmp(pointer, data, self->itemSize);

		if (result == 0)
			vector_remove(self, i);
	}
}

void
vector_sort(Vector* self, SortCompareFunction function)
{
	qsort(self->data, self->count, self->itemSize, function);
}
