#pragma once

#include "component_game_object.h"
#include "../physics/component_circle_collider.h"

static const RendererBuffer COMPONENT_PLANE_OBJECT_BUFFER = RENDERER_BUFFER_WORLD;
static const f32 COMPONENT_PLANE_OBJECT_FRICTION = 0.985f;
static const f32 COMPONENT_PLANE_OBJECT_DEPTH_Y_OFFSET_MULTIPLIER = 0.1f;
static const f32 COMPONENT_PLANE_OBJECT_DEPTH_MIN = 0.3f;
static const f32 COMPONENT_PLANE_OBJECT_DEPTH_MAX = 0.4f;
static const vec3 COMPONENT_PLANE_OBJECT_VELOCITY_MAX = {0.0f, 0.0f, 0.0f};
static const vec4 COMPONENT_PLANE_OBJECT_BOUNDS = {250.0f, 250.0f, 6300.0f, 3100.0f};

#define COMPONENT_PLANE_OBJECT_ORIGIN ORIGIN_CENTER

typedef struct ComponentPlaneObject
{
	Component component;
} ComponentPlaneObject;

void
component_plane_object_init
(
	ComponentPlaneObject* self, 
	ECS* ecs,
	Texture* texture, 
	vec2 size, 
	vec3 position,
	f32 radius
);

void component_plane_object_add(ComponentPlaneObject* self, ECS* ecs);
void component_plane_object_delete(ComponentPlaneObject* self, ECS* ecs);
void component_plane_object_tick(ComponentPlaneObject* self, ECS* ecs);

#define COMPONENT_PLANE_OBJECT_DEPENDENCY_COUNT 2
static const ComponentType COMPONENT_PLANE_OBJECT_DEPENDENCIES[COMPONENT_PLANE_OBJECT_DEPENDENCY_COUNT] = 
{
	COMPONENT_GAME_OBJECT,
	COMPONENT_CIRCLE_COLLIDER
};

static const ComponentInfo COMPONENT_PLANE_OBJECT_INFO = 
{
	.system = 
	{
		.functions = 
		{ 
			(ECSFunction)component_plane_object_add, 
			NULL,
			(ECSFunction)component_plane_object_tick, 
			NULL,
			NULL
		}
	},
	.type = COMPONENT_PLANE_OBJECT,
	.size = sizeof(ComponentPlaneObject)
};
