#pragma once

#include "../../ecs_entity.h"

#include "../../../../engine/rectangle.h"
#include "../../../../engine/font.h"
#include "../../../render/texture_quad.h"
#include "../../../resource/resource_shader.h"

#include "../physics/component_physics.h"

#include "../../entity/visual/entity_glyph.h"

#define COMPONENT_TEXT_LIMIT 1024
#define COMPONENT_TEXT_STRING_MAX 1024

#define COMPONENT_TEXT_COLOR_DEFAULT (f32*)OPAQUE
#define COMPONENT_TEXT_COLOR_COVER_DEFAULT (f32*)OPAQUE
#define COMPONENT_TEXT_DROP_SHADOW_COLOR_DEFAULT (f32*)BLACK
#define COMPONENT_TEXT_Z_OFFSET_INCREMENT 0.0001f

#define COMPONENT_TEXT_DROP_SHADOW_FONT_SIZE_MULTIPLIER 0.125f

static const vec2 COMPONENT_TEXT_SCALE_DEFAULT = {1.0f, 1.0f};
static const f32 COMPONENT_TEXT_WRAP_DEFAULT = -1;
static const vec3 COMPONENT_TEXT_OFFSET_DEFAULT = {0.0f, 0.0f, 0.0f};

typedef struct ComponentText
{
	Component component;
	Shader* shader;
	RendererBuffer buffer;
	Font* font;
	Vector glyphEntities; /* EntityID */
	char string[COMPONENT_TEXT_STRING_MAX];
	s32 wrap;
	u32 length;
	vec2 scale;
	vec3 offset;
	vec3 position;
	vec4 color;
	vec4 colorCover;
	vec4 dropShadowColor;
} ComponentText;

void component_text_add(ComponentText* self, ECS* ecs);
void component_text_delete(ComponentText* self, ECS* ecs);
void component_text_string_set(ComponentText* self, ECS* ecs, char* string, u32 length);
void component_text_tick(ComponentText* self, ECS* ecs);

void
component_text_init
(
	ComponentText* self, 
	ECS* ecs,
	Font* font,
	RendererBuffer buffer,
	char* string,
	vec3 position,
	u32 length,
	s32 wrap
);

static const ComponentInfo COMPONENT_TEXT_INFO = 
{
	.system = 
	{
		.functions = 
		{ 
			(ECSFunction)component_text_add, 
			(ECSFunction)component_text_delete, 
			(ECSFunction)component_text_tick,
			NULL,
			NULL
		}
	},
	.type = COMPONENT_TEXT,
	.size = sizeof(ComponentText)
};
