#pragma once

#include "../../ecs_entity.h"

#include "../../../../engine/rectangle.h"
#include "../../../render/texture_quad.h"
#include "../../../resource/resource_shader.h"

typedef struct ComponentTextureQuad
{
	Component component;
	Flip flip;
	Origin origin;
	RendererBuffer buffer;
	Texture* texture;
	f32 rotation;
	vec2 scale;
	vec2 size;
	vec2 uvMax;
	vec2 uvMin;
	vec3 offset;
	vec3 position;
	vec4 color;
	vec4 colorCover;
} ComponentTextureQuad;

void component_texture_quad_init
(
	ComponentTextureQuad* self, 
	ECS* ecs, 
	Texture* texture,
	Flip flip,
	Origin origin,
	RendererBuffer buffer,
	f32 rotation,
	vec2 size, 
	vec2 scale,
	vec2 uvMin, 
	vec2 uvMax, 
	vec3 position,
	vec3 offset,
	vec4 color
);

void component_texture_quad_size_get(ComponentTextureQuad* self, vec2 size);
void component_texture_quad_position_get(ComponentTextureQuad* self, vec3 position);
void component_texture_quad_origin_point_get(ComponentTextureQuad* self, vec3 originPoint);
void component_texture_quad_model_get(ComponentTextureQuad* self, mat4 model);
void component_texture_quad_rectangle_get(ComponentTextureQuad* self, Rectangle* rectangle);
s32 component_texture_quad_sort(ComponentTextureQuad* a, ComponentTextureQuad* b);
void component_texture_quad_draw(ComponentTextureQuad* self, ECS* ecs);

static const ComponentInfo COMPONENT_TEXTURE_QUAD_INFO = 
{
	.system = 
	{
		.functions = 
		{ 
			NULL,
			NULL, 
			NULL,
			NULL,
			(ECSFunction)component_texture_quad_draw
		}
	},
	.type = COMPONENT_TEXTURE_QUAD,
	.size = sizeof(ComponentTextureQuad)
};

#define COMPONENT_TEXTURE_QUAD_UV_MIN_DEFAULT TEXTURE_QUAD_UV_MIN_DEFAULT
#define COMPONENT_TEXTURE_QUAD_UV_MAX_DEFAULT TEXTURE_QUAD_UV_MAX_DEFAULT
#define COMPONENT_TEXTURE_QUAD_COLOR_DEFAULT TEXTURE_QUAD_COLOR_DEFAULT
#define COMPONENT_TEXTURE_QUAD_FLIP_DEFAULT TEXTURE_QUAD_FLIP_DEFAULT
#define COMPONENT_TEXTURE_QUAD_ORIGIN_DEFAULT TEXTURE_QUAD_ORIGIN_DEFAULT

static const RendererBuffer COMPONENT_TEXTURE_QUAD_BUFFER_DEFAULT = RENDERER_BUFFER_WORLD;
static const ShaderType COMPONENT_TEXTURE_QUAD_SHADER_DEFAULT = SHADER_TEXTURE_QUAD;
static const TextureType COMPONENT_TEXTURE_QUAD_TEXTURE_DEFAULT = TEXTURE_DEFAULT;
static const f32 COMPONENT_TEXTURE_QUAD_ROTATION_DEFAULT = 0.0f;
static const vec2 COMPONENT_TEXTURE_QUAD_SCALE_DEFAULT = {1.0f, 1.0f};
static const vec2 COMPONENT_TEXTURE_QUAD_SIZE_DEFAULT = {32.0f, 32.0f};
static const vec3 COMPONENT_TEXTURE_QUAD_POSITION_DEFAULT = {0.0f, 0.0f, 0.0f};
static const vec3 COMPONENT_TEXTURE_QUAD_OFFSET_DEFAULT = {0.0f, 0.0f, 0.0f};
static const vec3 COMPONENT_TEXTURE_QUAD_ROTATION_AXIS = {0.0f, 0.0f, 1.0f};
static const vec4 COMPONENT_TEXTURE_QUAD_COLOR_COVER = {0.0f, 0.0f, 0.0f, 1.0f};
