#include "component_value_text.h"

s32 _component_value_text_value_get(ComponentValueText* self, ECS* ecs);
s32 _component_value_text_max_get(ComponentValueText* self, ECS* ecs);
void _component_value_text_set(ComponentValueText* self, ECS* ecs);

s32
_component_value_text_value_get(ComponentValueText* self, ECS* ecs)
{
	u8* component;
	s32 value;

	component = (u8*)ecs_component_get(ecs, self->type, self->id);

	// Get first value of component, after the component struct. If it segfaults, not my problem.
	memcpy(&value, component + sizeof(Component), sizeof(u32));

	return value;
}

s32
_component_value_text_max_get(ComponentValueText* self, ECS* ecs)
{
	u8* component;
	s32 max;

	component = (u8*)ecs_component_get(ecs, self->type, self->id);

	// Get second value of component, after the component struct and the value. If it segfaults, not my problem.
	memcpy(&max, component + sizeof(Component) + sizeof(u32), sizeof(u32));

	return max;
}

void
_component_value_text_set(ComponentValueText* self, ECS* ecs)
{
	ComponentText* text;
	u32 minutes;
	u32 seconds;
	u32 milliseconds;
	u32 max;

	char string[COMPONENT_VALUE_TEXT_STRING_MAX];

	text = ecs_component_get(ecs, COMPONENT_TEXT, self->component.id);

	memset(string, '\0', COMPONENT_VALUE_TEXT_STRING_MAX);

	switch (self->textType)
	{
		case COMPONENT_VALUE_TEXT_TYPE_TIMER:
			time_formatted_get(self->value, &minutes, &seconds, &milliseconds);
			snprintf(string, COMPONENT_VALUE_TEXT_STRING_MAX, self->format, minutes, seconds, milliseconds);
			break;
		case COMPONENT_VALUE_TEXT_TYPE_INTEGER_MAX:
			max = _component_value_text_max_get(self, ecs);
			snprintf(string, COMPONENT_VALUE_TEXT_STRING_MAX, self->format, self->value, max);
			break;
		case COMPONENT_VALUE_TEXT_TYPE_INTEGER:
		default:
			snprintf(string, COMPONENT_VALUE_TEXT_STRING_MAX, self->format, self->value);
			break;
	}

	component_text_string_set(text, ecs, string, strlen(string));
}

void
component_value_text_add(ComponentValueText* self, ECS* ecs)
{
	ecs_components_add
	(
		ecs, 
		COMPONENT_VALUE_TEXT_DEPENDENCIES, 
		COMPONENT_VALUE_TEXT_DEPENDENCY_COUNT, 
		self->component.id
	);
}

void
component_value_text_tick(ComponentValueText* self, ECS* ecs)
{
	self->value = _component_value_text_value_get(self, ecs);

	if (self->value != self->previousValue)
	{
		_component_value_text_set(self, ecs);
		self->previousValue = self->value;
	}
}


void
component_value_text_init
(
	ComponentValueText* self, 
	ECS* ecs,
	Font* font,
	RendererBuffer buffer,
	const vec3 position,
	ComponentType type,
	ComponentValueTextType textType,
	const char* format,
	EntityID id
)
{
	component_text_init
	(
		ecs_component_get(ecs, COMPONENT_TEXT, self->component.id), 
		ecs, 
		font,
		buffer,
		"1",
		position,
		0,
		-1
	);

	self->format = format;
	self->type = type;
	self->textType = textType;
	self->id = id;
	self->value = _component_value_text_value_get(self, ecs);
	self->previousValue = self->value;
	
	_component_value_text_set(self, ecs);
}
