#include "ecs.h"

static void _ecs_function(ECS* self, ECSFunctionType type);

static void
_ecs_function(ECS* self, ECSFunctionType type)
{
	for (s32 i = 0; i < COMPONENT_COUNT; i++)
	{
		ComponentList* list;

		list = &self->lists[i];

		ecs_component_list_function(list, type);
	}
}

void
ecs_tick(ECS* self)
{
	if (!self->isFunctionDisabled[ECS_FUNCTION_TICK])
		_ecs_function(self, ECS_FUNCTION_TICK);
}

void
ecs_update(ECS* self)
{
	if (!self->isFunctionDisabled[ECS_FUNCTION_UPDATE])
		_ecs_function(self, ECS_FUNCTION_UPDATE);
}

void
ecs_draw(ECS* self)
{
	vector_sort(&self->lists[COMPONENT_TEXTURE_QUAD].components, (SortCompareFunction)component_texture_quad_sort);

	if (!self->isFunctionDisabled[ECS_FUNCTION_DRAW])
		_ecs_function(self, ECS_FUNCTION_DRAW);
}

void
ecs_init(ECS* self, Renderer* renderer, Control* control, Input* input, Resources* resources, Postprocessing* postprocessing)
{
	self->renderer = renderer;
	self->control = control;
	self->input = input;
	self->resources = resources;
	self->postprocessing = postprocessing;
	self->isLog = true;
	
	for (s32 i = 0 ; i < COMPONENT_COUNT; i++)
		ecs_component_list_init(&self->lists[i], self, COMPONENT_INFO[i]);
}

void
ecs_clear(ECS* self)
{
	for (s32 i = 0 ; i < COMPONENT_COUNT; i++)
		ecs_component_list_clear(&self->lists[i]);

	self->nextID = 0;
}

void
ecs_free(ECS* self)
{
	for (s32 i = 0 ; i < COMPONENT_COUNT; i++)
		ecs_component_list_free(&self->lists[i]);

	memset(self, '\0', sizeof(ECS));
}
