#include "ecs_component.h"

static bool isFirstTimeComponentAdded = false;

void*
ecs_component_get(ECS* self, ComponentType type, EntityID id)
{
	void* component;

	component = ecs_component_list_get(&self->lists[type], id);

	if 
	(
		(!component && !isFirstTimeComponentAdded) &&
		self->isLog
	)
		printf("%s Type: %i, ID: %u\n", STRING_ECS_COMPONENT_ID_ERROR, type, id);

	return component;
}

void*
ecs_component_from_index_get(ECS* self, ComponentType type, u32 index)
{
	void* component;

	component = vector_get(&self->lists[type].components, index);
 
	if 
	(
		(!component && !isFirstTimeComponentAdded) &&
		self->isLog
	)
		printf("%s Type: %i, ID: %i\n", STRING_ECS_COMPONENT_INDEX_ERROR, type, index);

	return component;
}

void*
ecs_component_add(ECS* self, ComponentType type, EntityID id)
{
	void* component;
	ECSFunction add;

	isFirstTimeComponentAdded = true;
	
	component = ecs_component_get(self, type, id);

	if (component)
		return component;

	isFirstTimeComponentAdded = false;

	component = vector_push(&self->lists[type].components, NULL);

	memcpy(component, (void*)&id, sizeof(EntityID));

	add = self->lists[type].system.functions[ECS_FUNCTION_ADD];

	if (add)
		add(component, self);

	return component;
}

void
ecs_component_delete(ECS* self, ComponentType type, EntityID id)
{
	for (s32 i = 0; i < (s32)self->lists[type].components.count; i++)
	{
		EntityID checkID;
		void* component;

		component = vector_get(&self->lists[type].components, i);

		memcpy(&checkID, component, sizeof(EntityID));

		if (checkID == id)
		{
			ECSFunction delete;

			delete = self->lists[type].system.functions[ECS_FUNCTION_DELETE];

			if (delete)
				delete(component, self);

			vector_remove(&self->lists[type].components, i);
		}
	}
}

void
ecs_components_add(ECS* self, const ComponentType* types, u32 count, EntityID id)
{
	for (s32 i = 0; i < (s32)count; i++)
		ecs_component_add(self, types[i], id);
}

void
ecs_components_delete(ECS* self, const ComponentType* types, u32 count, EntityID id)
{
	for (s32 i = 0; i < (s32)count; i++)
		ecs_component_delete(self, types[i], id);
}
