#pragma once

#include "../../ecs/entity/ui/entity_cursor.h"
#include "../../ecs/entity/ui/entity_meter.h"
#include "../../ecs/entity/ui/entity_medal.h"
#include "../../ecs/entity/visual/entity_value_text.h"
#include "../../ecs/entity/game/entity_player.h"
#include "../../ecs/entity/game/entity_foliage.h"
#include "../../ecs/entity/game/entity_target.h"
#include "../../ecs/entity/game/entity_collectible_amulet.h"
#include "../../ecs/entity/game/entity_collectible_stamina.h"
#include "../../ecs/entity/game/entity_collectible_power.h"
#include "../../ecs/entity/game/entity_smoke_big.h"
#include "../../ecs/entity/game/entity_collectible_food.h"
#include "../../ecs/entity/game/entity_collectible_rotten_food.h"
#include "../../ecs/entity/game/entity_trap.h"
#include "../../ecs/entity/utility/entity_timer.h"
#include "../../ecs/entity/visual/entity_dialogue.h"
#include "../../ecs/entity/visual/entity_text_disappearing.h"
#include "../../ecs/entity/visual/entity_text_queue_entry.h"

static const vec4 LEVEL_SUNSET_COLOR = {0.35f, 0.20f, 0.20f, 1.0f};

#define LEVEL_TUTORIAL_TEXT_COUNT 3

#define LEVEL_COUNTDOWN_COUNT (LEVEL_COUNTDOWN_GO + 1)
typedef enum LevelCountdownState
{
	LEVEL_COUNTDOWN_READY,
	LEVEL_COUNTDOWN_SET,
	LEVEL_COUNTDOWN_GO
} LevelCountdownState;

#define LEVEL_MEDAL_COUNT (LEVEL_MEDAL_GOLD + 1)
typedef enum LevelMedal
{
	LEVEL_MEDAL_NONE,
	LEVEL_MEDAL_BRONZE,
	LEVEL_MEDAL_SILVER,
	LEVEL_MEDAL_GOLD
} LevelMedal;

#define LEVEL_EVENT_COUNT (LEVEL_EVENT_ANCHOR_ARMS + 1)
typedef enum LevelEvent
{
	LEVEL_EVENT_NONE,
	LEVEL_EVENT_ROTTEN_FOOD,
	LEVEL_EVENT_STICKY_TRAPS,
	LEVEL_EVENT_BUTTERFINGERS,
	LEVEL_EVENT_OUT_OF_BREATH,
	LEVEL_EVENT_POWER_LEECH,
	LEVEL_EVENT_VIM_AND_VIGOR,
	LEVEL_EVENT_GLUTTONY,
	LEVEL_EVENT_ANCHOR_ARMS
} LevelEvent;

typedef struct LevelSettings
{
	u32 medalThresholds[LEVEL_MEDAL_COUNT - 1];
	u32 time;
	u32 eventTime;
	vec4 colorCover;
	u32 collectibleMax;
	u64 seedStart;
	bool isPower;
	bool isEvent;
	bool isAmulet;
	bool isSunset;
	bool isPowerForesight;
	bool isPowerCancel;
	bool isPowerMagnet;
	bool isPowerExtend;
	bool isPowerInvigorate;
	bool isTutorialMove;
	bool isTutorialStamina;
	bool isTutorialFood;
	bool isTutorialEvent;
	bool isTutorialPower;
	bool isTutorialPowerForesight;
	bool isTutorialPowerCancel;
	bool isTutorialPowerMagnet;
	bool isTutorialPowerExtend;
	bool isTutorialPowerInvigorate;
	bool isTutorialFinal;
} LevelSettings;

typedef struct Level
{
	ECS* ecs;
	LevelSettings settings;
	LevelCountdownState countdownState;
	LevelMedal medal;
	s32 score;
	s32 timeToMedal;
	s32 timerValue;
	u32 levelValue;
	u64 eventSeed;
	u64 collectibleSeed;
	LevelMedal nextMedal;
	LevelEvent event;
	LevelEvent nextEvent;
	Vector textQueue;
	Vector eventQueue;
	EntityID plane;
	EntityID bg;
	EntityID countdownTimer;
	EntityID cursor;
	EntityID finishTimer;
	EntityID medalUI;
	EntityID meterPower;
	EntityID meterStamina;
	EntityID pauseText;
	EntityID pauseTipText;
	EntityID player;
	EntityID playerFoodUI;
	EntityID target;
	EntityID targetFoodUI;
	EntityID thresholdUI;
	EntityID collectibleMercyTimer;
	EntityID timer;
	EntityID timerUI;
	EntityID eventTimer;
	EntityID eventTimerUI;
	EntityID amulet;
	bool isAmuletSpawn;
	bool isRottenFood;
	bool isMagnet;
	bool isComplete;
	bool isCountdownDone;
	bool isPaused;
	bool isTimerDone;
	bool isFinished;
} Level;
