#include "countdown.h"

void 
level_countdown_init(Level* self)
{
	ComponentPlayerControl* playerControl;
	ComponentTimer* levelTimer;

	playerControl = ecs_component_get(self->ecs, COMPONENT_PLAYER_CONTROL, self->player);
	levelTimer = ecs_component_get(self->ecs, COMPONENT_TIMER, self->timer);

	playerControl->component.isDisabled = true;
	levelTimer->component.isDisabled = true;

	self->countdownState = LEVEL_COUNTDOWN_READY;

	self->countdownTimer = entity_timer_add(self->ecs, LEVEL_COUNTDOWN_TIMER_VALUE);

	level_text_queue_add(self, LEVEL_STRING_RESET_REMINDER);
	level_text_queue_add(self, LEVEL_STRING_COUNTDOWN_SKIP);

	entity_text_disappearing_add
	(
		self->ecs,
		&self->ecs->resources->fonts[FONT_BIG],
		RENDERER_BUFFER_UI,
		STRING_LEVEL_COUNTDOWN_READY,
		LEVEL_COUNTDOWN_TEXT_POSITION,
		-1,
		LEVEL_COUNTDOWN_TIMER_VALUE
	);
}

void
level_countdown_finish(Level* self)
{
	ComponentPlayerControl* playerControl;
	ComponentTimer* levelTimer;
	ComponentTimer* countdownTimer;

	playerControl = ecs_component_get(self->ecs, COMPONENT_PLAYER_CONTROL, self->player);
	levelTimer = ecs_component_get(self->ecs, COMPONENT_TIMER, self->timer);
	countdownTimer = ecs_component_get(self->ecs, COMPONENT_TIMER, self->countdownTimer);

	playerControl->component.isDisabled = false;
	levelTimer->component.isDisabled = false;

	countdownTimer->isFinished = true;
	countdownTimer->value = 0;
	self->countdownState = LEVEL_COUNTDOWN_GO;
	self->isCountdownDone = true;

	level_text_queue_clear(self);
	level_tutorial_init(self);

	if (self->settings.isEvent)
	{
		ComponentTimer* eventTimer;

		level_event_choose(self);

		eventTimer = ecs_component_get(self->ecs, COMPONENT_TIMER, self->eventTimer);

		eventTimer->component.isDisabled = false;
	}

	entity_text_disappearing_add
	(
		self->ecs,
		&self->ecs->resources->fonts[FONT_BIG],
		RENDERER_BUFFER_UI,
		STRING_LEVEL_COUNTDOWN_GO,
		LEVEL_COUNTDOWN_TEXT_POSITION,
		-1,
		LEVEL_COUNTDOWN_TIMER_VALUE
	);



	music_play(&self->ecs->resources->music[MUSIC_GAME], true);
}

void 
level_countdown_tick(Level* self)
{
	ComponentTimer* countdownTimer;

	if (self->countdownState == LEVEL_COUNTDOWN_GO)
		return;

	countdownTimer = ecs_component_get(self->ecs, COMPONENT_TIMER, self->countdownTimer);

	if (countdownTimer->isFinished)
	{
		self->countdownState++;

		if (self->countdownState == LEVEL_COUNTDOWN_GO)
		{
			level_countdown_finish(self);
			return;
		}

		component_timer_init(countdownTimer, self->ecs, LEVEL_COUNTDOWN_TIMER_VALUE);
	}

	if (control_pressed(self->ecs->control, CONTROL_ACTION_TERTIARY) && !self->isPaused)
	{
		level_countdown_finish(self);
		return;
	}
}
