#include "pause.h"

void
level_pause_init(Level* self)
{
	self->isPaused = false;
}

void 
level_pause(Level* self)
{
	ComponentText* pauseText;
	ComponentText* pauseTipText;

	self->pauseText = entity_text_add
	(
		self->ecs,
		&self->ecs->resources->fonts[FONT_BIG], 
		RENDERER_BUFFER_UI,
		STRING_LEVEL_PAUSE,
		LEVEL_PAUSE_TEXT_POSITION,
		-1
	);

	self->pauseTipText = entity_text_add
	(
		self->ecs,
		&self->ecs->resources->fonts[FONT_SMALL], 
		RENDERER_BUFFER_UI,
		STRING_LEVEL_PAUSE_TIP,
		LEVEL_PAUSE_TIP_TEXT_POSITION,
		-1
	);

	pauseText = ecs_component_get(self->ecs, COMPONENT_TEXT, self->pauseText);
	pauseTipText = ecs_component_get(self->ecs, COMPONENT_TEXT, self->pauseTipText);

	glm_vec4_copy(OPAQUE, pauseText->color);
	glm_vec4_copy(OPAQUE, pauseTipText->color);

	component_text_tick(pauseText, self->ecs);
	component_text_tick(pauseTipText, self->ecs);

	self->ecs->isFunctionDisabled[ECS_FUNCTION_TICK] = true;
	self->ecs->isFunctionDisabled[ECS_FUNCTION_UPDATE] = true;
	
	glm_vec3_copy(LEVEL_PAUSE_COLOR, self->ecs->postprocessing[RENDERER_BUFFER_PLANE].color);

	if (self->settings.isSunset)
		glm_vec3_copy(BLACK, self->ecs->postprocessing[RENDERER_BUFFER_WORLD].color);
	else
		glm_vec3_copy(LEVEL_PAUSE_COLOR, self->ecs->postprocessing[RENDERER_BUFFER_WORLD].color);

	glm_vec3_copy(LEVEL_PAUSE_COLOR, self->ecs->postprocessing[RENDERER_BUFFER_BACKGROUND].color);
	glm_vec3_copy(LEVEL_PAUSE_COLOR, self->ecs->postprocessing[RENDERER_BUFFER_OVERLAY].color);
}

void 
level_unpause(Level* self)
{
	ComponentText* pauseText;
	ComponentText* pauseTipText;

	if (self->isPaused)
	{
		ecs_entity_delete(self->ecs, self->pauseText);
		ecs_entity_delete(self->ecs, self->pauseTipText);
	}

	self->ecs->isFunctionDisabled[ECS_FUNCTION_TICK] = false;
	self->ecs->isFunctionDisabled[ECS_FUNCTION_UPDATE] = false;
	
	glm_vec3_copy(OPAQUE, self->ecs->postprocessing[RENDERER_BUFFER_PLANE].color);
	
	if (self->settings.isSunset)
		glm_vec3_copy(BLACK, self->ecs->postprocessing[RENDERER_BUFFER_WORLD].color);
	else
		glm_vec3_copy(OPAQUE, self->ecs->postprocessing[RENDERER_BUFFER_WORLD].color);

	glm_vec3_copy(OPAQUE, self->ecs->postprocessing[RENDERER_BUFFER_BACKGROUND].color);
	glm_vec3_copy(OPAQUE, self->ecs->postprocessing[RENDERER_BUFFER_OVERLAY].color);
}

void 
level_pause_tick(Level* self)
{
	if (control_pressed(self->ecs->control, CONTROL_PAUSE))
	{
		if (!self->isPaused)
		{
			level_pause(self);
			sound_play(&self->ecs->resources->sounds[SOUND_PAUSE]);
		}
		else
		{
			level_unpause(self);
			sound_play(&self->ecs->resources->sounds[SOUND_UNPAUSE]);
		}

		self->isPaused = !self->isPaused;
	}
}
